@@ -6,37 +6,71 @@ extern struct uwsgi_server uwsgi;
 extern char **environ;
 
 int emperor_queue;
+char *emperor_absolute_dir;
+
+struct uwsgi_instance {
+        struct uwsgi_instance *ui_prev;
+        struct uwsgi_instance *ui_next;
+
+        char name[0xff];
+        pid_t pid;
+
+        int status;
+        time_t born;
+        time_t last_mod;
+        time_t last_loyal;
+
+        uint64_t respawns;
+        int use_config;
+
+        int pipe[2];
+        int pipe_config[2];
+
+        char *config;
+        uint32_t config_len;
+
+        int loyal;
+
+        int zerg;
+};
+
+
+struct uwsgi_instance *ui;
 
 static void royal_death(int signum) {
+
+
+	if (uwsgi.vassals_stop_hook) {
+
+		struct uwsgi_instance *c_ui = ui->ui_next;
+
+		while (c_ui) {
+                        uwsgi_log("running vassal stop-hook: %s %s\n", uwsgi.vassals_stop_hook, c_ui->name);
+			if (setenv("UWSGI_VASSALS_DIR", emperor_absolute_dir, 1)) {
+                                uwsgi_error("setenv()");
+                        }
+                        int stop_hook_ret = uwsgi_run_command_and_wait(uwsgi.vassals_stop_hook, c_ui->name);
+                        uwsgi_log("%s stop-hook returned %d\n", c_ui->name, stop_hook_ret);
+			c_ui = c_ui->ui_next;
+		}
+        }
 	uwsgi_notify("The Emperor is buried.");
 	exit(0);
 }
 
-struct uwsgi_instance {
-	struct uwsgi_instance *ui_prev;
-	struct uwsgi_instance *ui_next;
 
-	char name[0xff];
-	pid_t pid;
+void emperor_stats() {
 
-	int status;
-	time_t born;
-	time_t last_mod;
-	time_t last_loyal;
+	struct uwsgi_instance *c_ui = ui->ui_next;
 
-	uint64_t respawns;
-	int use_config;
+        while (c_ui) {
 
-	int pipe[2];
-	int pipe_config[2];
+		uwsgi_log("vassal instance %s (last modified %d) status %d loyal %d zerg %d\n",c_ui->name, c_ui->last_mod, c_ui->status, c_ui->loyal, c_ui->zerg);
 
-	char *config;
-	uint32_t config_len;
+                c_ui = c_ui->ui_next;
+        }
 
-	int loyal;
-};
-
-struct uwsgi_instance *ui;
+}
 
 struct uwsgi_instance *emperor_get_by_fd(int fd) {
 
@@ -86,12 +120,19 @@ void emperor_del(struct uwsgi_instance *c_ui) {
 
 	if (uwsgi.vassals_stop_hook) {
                         uwsgi_log("running vassal stop-hook: %s %s\n", uwsgi.vassals_stop_hook, c_ui->name);
+			if (setenv("UWSGI_VASSALS_DIR", emperor_absolute_dir, 1)) {
+                                uwsgi_error("setenv()");
+                        }
                         int stop_hook_ret = uwsgi_run_command_and_wait(uwsgi.vassals_stop_hook, c_ui->name);
                         uwsgi_log("%s stop-hook returned %d\n", c_ui->name, stop_hook_ret);
                 }
 
 	uwsgi_log("removed uwsgi instance %s\n", c_ui->name);
 
+	if (c_ui->zerg) {
+		uwsgi.emperor_broodlord_count--;
+	}
+
 	free(c_ui);
 
 }
@@ -137,6 +178,7 @@ void emperor_add(char *name, time_t born, char *config, uint32_t config_size) {
 	char *uef;
 	char **uenvs;
 	int counter;
+	char *colon = NULL;
 	int i;
 
 	sleep(1);
@@ -159,6 +201,12 @@ void emperor_add(char *name, time_t born, char *config, uint32_t config_size) {
 	uwsgi_log("c_ui->ui_next = %p\n", c_ui->ui_next);
 #endif
 	n_ui->ui_prev = c_ui;
+
+	if (strchr(name,':')) {
+		n_ui->zerg = 1;
+		uwsgi.emperor_broodlord_count++;
+	}
+
 	memcpy(n_ui->name, name, strlen(name));
 	n_ui->born = born;
 	n_ui->last_mod = born;
@@ -263,6 +311,13 @@ void emperor_add(char *name, time_t born, char *config, uint32_t config_size) {
 		argv = uwsgi_malloc(sizeof(char *) * counter);
 		// set args
 		argv[0] = uwsgi.binary_path;
+
+		if (uwsgi.emperor_broodlord) {
+			colon = strchr(name, ':');
+			if (colon) {
+				colon[0] = 0;
+			}
+		}
 		if (!strcmp(name + (strlen(name) - 4), ".xml"))
 			argv[1] = "--xml";
 		if (!strcmp(name + (strlen(name) - 4), ".ini"))
@@ -273,6 +328,10 @@ void emperor_add(char *name, time_t born, char *config, uint32_t config_size) {
 			argv[1] = "--yaml";
 		if (!strcmp(name + (strlen(name) - 3), ".js"))
 			argv[1] = "--json";
+
+		if (colon) {
+			colon[0] = ':';
+		}
 		argv[2] = name;
 		counter = 3;
 		uct = uwsgi.vassals_templates;
@@ -293,6 +352,9 @@ void emperor_add(char *name, time_t born, char *config, uint32_t config_size) {
 
 		if (uwsgi.vassals_start_hook) {
 			uwsgi_log("running vassal start-hook: %s %s\n", uwsgi.vassals_start_hook, n_ui->name);
+			if (setenv("UWSGI_VASSALS_DIR", emperor_absolute_dir, 1)) {
+				uwsgi_error("setenv()");
+			}
 			int start_hook_ret = uwsgi_run_command_and_wait(uwsgi.vassals_start_hook, n_ui->name);
 			uwsgi_log("%s start-hook returned %d\n", n_ui->name, start_hook_ret);
 		}
@@ -341,6 +403,8 @@ void emperor_loop() {
 	signal(SIGPIPE, SIG_IGN);
 	uwsgi_unix_signal(SIGINT, royal_death);
         uwsgi_unix_signal(SIGTERM, royal_death);
+        uwsgi_unix_signal(SIGQUIT, royal_death);
+        uwsgi_unix_signal(SIGUSR1, emperor_stats);
 
 	memset(&ui_base, 0, sizeof(struct uwsgi_instance));
 
@@ -395,6 +459,7 @@ reconnect:
 			uwsgi_error("glob()");
 			exit(1);
 		}
+		emperor_absolute_dir = realpath(".", NULL);
 	}
 
 	ui = &ui_base;
@@ -512,6 +577,15 @@ reconnect:
 							
 							// TODO post-start hook
 						}
+						else if (byte == 22) {
+							emperor_stop(ui_current);
+						}
+						else if (byte == 30 && uwsgi.emperor_broodlord > 0 && uwsgi.emperor_broodlord_count < uwsgi.emperor_broodlord) {
+							uwsgi_log("[emperor] going in broodlord mode: launching zergs for %s\n", ui_current->name);
+							char *zerg_name = uwsgi_concat3(ui_current->name,":","zerg");
+							emperor_add(zerg_name, time(NULL), NULL, 0);
+							free(zerg_name);
+						}
 					}
 				}
 				else {
@@ -643,7 +717,7 @@ reconnect:
 				emperor_del(ui_current);
 				break;
 			}
-			else if (!ui_current->use_config && strncmp(ui_current->name, "http://",7) && stat(ui_current->name, &st)) {
+			else if (!ui_current->use_config && strncmp(ui_current->name, "http://",7) && stat(ui_current->name, &st) && !ui_current->zerg) {
 				emperor_stop(ui_current);
 			}
 			else if (ui_current->pid == diedpid) {
