@@ -3,7 +3,11 @@
 extern char **environ;
 extern struct uwsgi_server uwsgi;
 
+#ifdef __APPLE__
+extern struct uwsgi_perl uperl;
+#else
 struct uwsgi_perl uperl;
+#endif
 
 struct option uwsgi_perl_options[] = {
 
