@@ -5,9 +5,8 @@ extern struct uwsgi_server uwsgi;
 
 int uwsgi_rpc_request(struct wsgi_request *wsgi_req) {
 
-	char *argv[0xff];
-	uint8_t argc;
-	int i;
+	char *argv[256];
+	uint8_t argc = 0xff;
 
 	/* Standard RPC request */
         if (!wsgi_req->uh.pktsize) {
@@ -15,17 +14,25 @@ int uwsgi_rpc_request(struct wsgi_request *wsgi_req) {
                 return -1;
         }
 
-	uwsgi_log("parsing array\n");
+	/*
+	for(argc=0;argc<wsgi_req->uh.pktsize;argc++) {
+		uwsgi_log("rpc: %c\n", wsgi_req->buffer[argc]);
+	}
+	*/
+
+#ifdef UWSGI_DEBUG
+	uwsgi_log("RPC pktsize %d\n", wsgi_req->uh.pktsize);
+#endif
+
 	if (uwsgi_parse_array(wsgi_req->buffer, wsgi_req->uh.pktsize, argv, &argc)) {
                 uwsgi_log("Invalid RPC request. skip.\n");
                 return -1;
 	}
-	uwsgi_log("done\n");
-	
-	for(i=0;i<argc;i++) {
-		uwsgi_log("arg %d %s\n", i, argv[i]);
-	}
 
+#ifdef UWSGI_DEBUG
+	uwsgi_log("RPC args %d\n", argc-1);
+#endif
+	
 	wsgi_req->uh.pktsize = uwsgi_rpc(argv[0], argc-1, argv+1, wsgi_req->buffer);
 
 	if (wsgi_req->uh.modifier2 == 0) {
